"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LocaleProvider = void 0;
var React = require("react");
var LocaleContext_1 = require("./LocaleContext");
var localizer_1 = require("./localizer");
var LocaleProvider = (function (_super) {
    __extends(LocaleProvider, _super);
    function LocaleProvider() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            language: '',
        };
        _this.setLanguage = function (language) {
            var _a = _this.props, languages = _a.languages, localizeUrls = _a.localizeUrls;
            var doesLanguageExist = languages.indexOf(language.toLowerCase()) > -1;
            if (doesLanguageExist) {
                _this.setState({
                    language: language.toLowerCase(),
                }, function () {
                    if (localizeUrls) {
                        _this.updateUrl();
                    }
                    else {
                        _this.updateBrowserStorage();
                    }
                });
            }
            else {
                console.error('⛔ Selected language does not exist in the initial <LocaleProvider/> configuration!');
            }
        };
        _this.getLanguageFromUrl = function () {
            var languages = _this.props.languages;
            if (typeof window === 'undefined') {
                return null;
            }
            return localizer_1.getLanguageFromUrl(languages, window.location.pathname);
        };
        _this.updateUrl = function () {
            var e_1, _a;
            var _b = _this.props, languages = _b.languages, defaultLanguage = _b.defaultLanguage, history = _b.history;
            var currentLanguage = _this.state.language;
            var currentUrl = location.pathname + location.search;
            var searchString = defaultLanguage || languages[0];
            try {
                for (var languages_1 = __values(languages), languages_1_1 = languages_1.next(); !languages_1_1.done; languages_1_1 = languages_1.next()) {
                    var language = languages_1_1.value;
                    if (currentUrl.includes(language)) {
                        searchString = language;
                        break;
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (languages_1_1 && !languages_1_1.done && (_a = languages_1.return)) _a.call(languages_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
            var updatedUrl = currentUrl.replace("/" + searchString + "/", "/" + currentLanguage + "/");
            history === null || history === void 0 ? void 0 : history.push(updatedUrl);
        };
        _this.performTranslation = function (id, vars) {
            var language = _this.state.language;
            var translations = _this.props.translations;
            if (!id) {
                return "\u26D4 Missing translation key";
            }
            if (!translations) {
                _this.handleError('NO_CONFIG', language, id);
                return "\u26D4 " + id.toUpperCase();
            }
            else if (!translations[id]) {
                _this.handleError('NO_KEY', language, id);
                return "\u26D4 " + id.toUpperCase();
            }
            else if (!translations[id][language]) {
                _this.handleError('NO_KEY', language, id);
                return "\u26D4 " + id.toUpperCase() + " - " + language.toUpperCase();
            }
            else {
                var translatedText = translations[id][language];
                if (vars) {
                    return _this.processVars(id, translatedText, vars);
                }
                return translatedText;
            }
        };
        _this.performTranslationAndParse = function (id, vars) {
            var translatedText = _this.performTranslation(id, vars);
            return React.createElement("span", { dangerouslySetInnerHTML: { __html: translatedText } });
        };
        return _this;
    }
    LocaleProvider.prototype.componentDidMount = function () {
        this.getIntialLanguage();
    };
    LocaleProvider.prototype.getIntialLanguage = function () {
        var _a = this.props, defaultLanguage = _a.defaultLanguage, languages = _a.languages, _b = _a.localizeUrls, localizeUrls = _b === void 0 ? false : _b;
        var urlLanguageSetting = this.getLanguageFromUrl();
        var browserLanguageSetting = this.getBrowserLanguage();
        var resolvedLanguage = this.getLanguageFromLocationState() || defaultLanguage || browserLanguageSetting || languages[0];
        if (localizeUrls && urlLanguageSetting === null) {
            this.insertLanguageToUrl(resolvedLanguage);
        }
        this.setLanguage(resolvedLanguage);
        return resolvedLanguage;
    };
    LocaleProvider.prototype.getBrowserLanguage = function () {
        if (typeof navigator === undefined) {
            return null;
        }
        return navigator.language.split(/[-_]/)[0];
    };
    LocaleProvider.prototype.getLanguageFromLocationState = function () {
        var _a = this.props.localizeUrls, localizeUrls = _a === void 0 ? false : _a;
        if (typeof window === 'undefined') {
            return null;
        }
        if (localizeUrls) {
            window.localStorage.removeItem('k2-lang');
            return this.getLanguageFromUrl();
        }
        else {
            return window.localStorage.getItem('k2-lang') || '';
        }
    };
    LocaleProvider.prototype.insertLanguageToUrl = function (language) {
        var history = this.props.history;
        var pathName = window.location.pathname + window.location.search;
        var url = "/" + (language + pathName);
        history === null || history === void 0 ? void 0 : history.push(url);
    };
    LocaleProvider.prototype.updateBrowserStorage = function () {
        var language = this.state.language;
        window.localStorage.setItem('k2-lang', language);
    };
    LocaleProvider.prototype.handleError = function (errorCode, language, id, varKey) {
        if (varKey === void 0) { varKey = ''; }
        var errorMap = {
            NO_CONFIG: '⛔ Translations not found!',
            NO_KEY: "\u26D4 Missing Translation key: " + id.toUpperCase(),
            NO_VAL: "\u26D4 Missing Translation to: " + id.toUpperCase() + " => " + language.toUpperCase(),
            NO_VAR: "\u26D4 Missing Variable { " + varKey + " : string } in: " + id.toUpperCase() + " => " + language.toUpperCase(),
        };
        var errorMessage = errorMap[errorCode];
        console.error(errorMessage);
    };
    LocaleProvider.prototype.compareSearchedAndReplacedString = function (searchedString, replacedString) {
        return searchedString.toLowerCase() === replacedString.toLowerCase();
    };
    LocaleProvider.prototype.replaceVars = function (textToReplaceVars, searchValue, replaceValue, id, varKey) {
        var searchRegex = new RegExp(searchValue, 'gi');
        var textWithReplacedVars = textToReplaceVars.replace(searchRegex, replaceValue);
        var isVarReplacementSuccessful = !this.compareSearchedAndReplacedString(textToReplaceVars, textWithReplacedVars);
        if (isVarReplacementSuccessful === false) {
            this.handleError('NO_VAR', varKey, id);
        }
        return textWithReplacedVars;
    };
    LocaleProvider.prototype.processVars = function (id, translatedText, vars) {
        var _this = this;
        if (vars === void 0) { vars = {}; }
        var varKeys = Object.keys(vars) || [];
        var parsedText = translatedText;
        varKeys.forEach(function (varKey) {
            var searchValue = "{" + varKey.trim() + "}";
            var replaceValue = vars[varKey].toString().trim();
            parsedText = _this.replaceVars(parsedText, searchValue, replaceValue, varKey, id);
        });
        return parsedText;
    };
    LocaleProvider.prototype.render = function () {
        var _a = this.props, translations = _a.translations, _b = _a.localizeUrls, localizeUrls = _b === void 0 ? false : _b, history = _a.history, children = _a.children;
        var language = this.state.language;
        if (localizeUrls && !history) {
            throw new Error('Missing history prop from LocaleProvider');
        }
        if (!translations) {
            throw new Error('Missing translations prop from LocaleProvider');
        }
        if (localizeUrls && history && language) {
            var Router = require('react-router-dom').Router;
            return (React.createElement(LocaleContext_1.LocaleContext.Provider, { value: {
                    language: language,
                    translations: translations,
                    changeLanguage: this.setLanguage,
                    performTranslation: this.performTranslation,
                    performTranslationAndParse: this.performTranslationAndParse,
                } },
                React.createElement(Router, { history: history }, children)));
        }
        if (!localizeUrls) {
            return (React.createElement(LocaleContext_1.LocaleContext.Provider, { value: {
                    language: language || this.getIntialLanguage(),
                    translations: translations,
                    changeLanguage: this.setLanguage,
                    performTranslation: this.performTranslation,
                    performTranslationAndParse: this.performTranslationAndParse,
                } }, children));
        }
        return null;
    };
    return LocaleProvider;
}(React.Component));
exports.LocaleProvider = LocaleProvider;
